@@ -20,6 +20,7 @@ import shlex
 import sys
 import time
 import timeit
+import signal
 from typing import Dict, Any
 from ast import (
     Assign,
@@ -69,17 +70,26 @@ from IPython.core.magics.ast_mod import ReplaceCodeTransformer
 #-----------------------------------------------------------------------------
 
 
-class TimeitResult(object):
+class TimeitResult:
     """
     Object returned by the timeit magic with info about the run.
 
-    Contains the following attributes :
+    Contains the following attributes:
 
-    loops: (int) number of loops done per measurement
-    repeat: (int) number of times the measurement has been repeated
-    best: (float) best execution time / number
-    all_runs: (list of float) execution time of each run (in s)
-    compile_time: (float) time of statement compilation (s)
+    loops: int
+      number of loops done per measurement
+
+    repeat: int
+      number of times the measurement was repeated
+
+    best: float
+      best execution time / number
+
+    all_runs : list[float]
+      execution time of each run (in s)
+
+    compile_time: float
+      time of statement compilation (s)
 
     """
     def __init__(self, loops, repeat, best, worst, all_runs, compile_time, precision):
@@ -90,7 +100,7 @@ class TimeitResult(object):
         self.all_runs = all_runs
         self.compile_time = compile_time
         self._precision = precision
-        self.timings = [ dt / self.loops for dt in all_runs]
+        self.timings = [dt / self.loops for dt in all_runs]
 
     @property
     def average(self):
@@ -105,8 +115,8 @@ class TimeitResult(object):
         pm = '+-'
         if hasattr(sys.stdout, 'encoding') and sys.stdout.encoding:
             try:
-                u'\xb1'.encode(sys.stdout.encoding)
-                pm = u'\xb1'
+                "\xb1".encode(sys.stdout.encoding)
+                pm = "\xb1"
             except:
                 pass
         return "{mean} {pm} {std} per loop (mean {pm} std. dev. of {runs} run{run_plural}, {loops:,} loop{loop_plural} each)".format(
@@ -121,7 +131,7 @@ class TimeitResult(object):
 
     def _repr_pretty_(self, p , cycle):
         unic = self.__str__()
-        p.text(u'<TimeitResult : '+unic+u'>')
+        p.text("<TimeitResult : " + unic + ">")
 
 
 class TimeitTemplateFiller(ast.NodeTransformer):
@@ -151,10 +161,11 @@ class TimeitTemplateFiller(ast.NodeTransformer):
 
 class Timer(timeit.Timer):
     """Timer class that explicitly uses self.inner
-    
+
     which is an undocumented implementation detail of CPython,
     not shared by PyPy.
     """
+
     # Timer.timeit copied from CPython 3.4.2
     def timeit(self, number=timeit.default_number):
         """Time 'number' executions of the main statement.
@@ -192,19 +203,16 @@ class ExecutionMagics(Magics):
     @no_var_expand
     @line_cell_magic
     def prun(self, parameter_s='', cell=None):
-
         """Run a statement through the python code profiler.
 
-        **Usage, in line mode:**
+        **Usage, in line mode**::
 
           %prun [options] statement
 
-        **Usage, in cell mode:**
+        **Usage, in cell mode**::
 
           %%prun [options] [statement]
-
           code...
-
           code...
 
         In cell mode, the additional code lines are appended to the (possibly
@@ -309,6 +317,7 @@ class ExecutionMagics(Magics):
             the magic line is always left unmodified.
 
         """
+        # TODO: port to magic_arguments as currently this is duplicated in IPCompleter._extract_code
         opts, arg_str = self.parse_options(parameter_s, 'D:l:rs:T:q',
                                            list_all=True, posix=False)
         if cell is not None:
@@ -433,11 +442,10 @@ class ExecutionMagics(Magics):
         Set break point at LINE in FILE.
         """
     )
-    @magic_arguments.argument('statement', nargs='*',
-        help="""
-        Code to run in debugger.
-        You can omit this in cell magic mode.
-        """
+    @magic_arguments.kwds(
+        epilog="""
+      Any remaining arguments will be treated as code to run in the debugger.
+    """
     )
     @no_var_expand
     @line_cell_magic
@@ -446,12 +454,12 @@ class ExecutionMagics(Magics):
         """Activate the interactive debugger.
 
         This magic command support two ways of activating debugger.
-        One is to activate debugger before executing code.  This way, you
+        One is to activate debugger before executing code. This way, you
         can set a break point, to step through the code from the point.
         You can use this mode by giving statements to execute and optionally
         a breakpoint.
 
-        The other one is to activate debugger in post-mortem mode.  You can
+        The other one is to activate debugger in post-mortem mode. You can
         activate this mode simply running %debug without any argument.
         If an exception has just occurred, this lets you inspect its stack
         frames interactively.  Note that this will always work only on the last
@@ -467,9 +475,9 @@ class ExecutionMagics(Magics):
             the magic line is always left unmodified.
 
         """
-        args = magic_arguments.parse_argstring(self.debug, line)
+        args, extra = magic_arguments.parse_argstring(self.debug, line, partial=True)
 
-        if not (args.breakpoint or args.statement or cell):
+        if not (args.breakpoint or extra or cell):
             self._debug_post_mortem()
         elif not (args.breakpoint or cell):
             # If there is no breakpoints, the line is just code to execute
@@ -478,7 +486,7 @@ class ExecutionMagics(Magics):
             # Here we try to reconstruct the code from the output of
             # parse_argstring. This might not work if the code has spaces
             # For example this fails for `print("a b")`
-            code = "\n".join(args.statement)
+            code = " ".join(extra)
             if cell:
                 code += "\n" + cell
             self._debug_exec(code, args.breakpoint, local_ns)
@@ -565,8 +573,8 @@ class ExecutionMagics(Magics):
         sees its environment as if it were being run as a stand-alone program
         (except for sharing global objects such as previously imported
         modules). But after execution, the IPython interactive namespace gets
-        updated with all variables defined in the program (except for __name__
-        and sys.argv). This allows for very convenient loading of code for
+        updated with all variables defined in the program (except for ``__name__``
+        and ``sys.argv``). This allows for very convenient loading of code for
         interactive work, while giving each program a 'clean sheet' to run in.
 
         Arguments are expanded using shell-like glob match.  Patterns
@@ -606,25 +614,25 @@ class ExecutionMagics(Magics):
           time.clock().  Under Unix, an estimate of time spent on system tasks
           is also given (for Windows platforms this is reported as 0.0).
 
-        If -t is given, an additional ``-N<N>`` option can be given, where <N>
-        must be an integer indicating how many times you want the script to
-        run.  The final timing report will include total and per run results.
+          If -t is given, an additional ``-N<N>`` option can be given, where <N>
+          must be an integer indicating how many times you want the script to
+          run.  The final timing report will include total and per run results.
 
-        For example (testing the script uniq_stable.py)::
+          For example (testing the script myscript.py)::
 
-            In [1]: run -t uniq_stable
+              In [1]: run -t myscript
 
-            IPython CPU timings (estimated):
-              User  :    0.19597 s.
-              System:        0.0 s.
+              IPython CPU timings (estimated):
+                User  :    0.19597 s.
+                System:        0.0 s.
 
-            In [2]: run -t -N5 uniq_stable
+              In [2]: run -t -N5 myscript
 
-            IPython CPU timings (estimated):
-            Total runs performed: 5
-              Times :      Total       Per run
-              User  :   0.910862 s,  0.1821724 s.
-              System:        0.0 s,        0.0 s.
+              IPython CPU timings (estimated):
+              Total runs performed: 5
+                Times :      Total       Per run
+                User  :   0.910862 s,  0.1821724 s.
+                System:        0.0 s,        0.0 s.
 
         -d
           run your program under the control of pdb, the Python debugger.
@@ -922,11 +930,6 @@ class ExecutionMagics(Magics):
             self.shell.InteractiveTB.pdb = self.shell.InteractiveTB.debugger_cls()
             deb = self.shell.InteractiveTB.pdb
 
-        # deb.checkline() fails if deb.curframe exists but is None; it can
-        # handle it not existing. https://github.com/ipython/ipython/issues/10028
-        if hasattr(deb, 'curframe'):
-            del deb.curframe
-
         # reset Breakpoint state, which is moronically kept
         # in a class
         bdb.Breakpoint.next = 1
@@ -993,7 +996,7 @@ class ExecutionMagics(Magics):
                     # Stop iteration is raised on quit command
                     pass
 
-        except:
+        except Exception:
             etype, value, tb = sys.exc_info()
             # Skip three frames in the traceback: the %run one,
             # one inside bdb.py, and the command-line typed by the
@@ -1046,16 +1049,14 @@ class ExecutionMagics(Magics):
     def timeit(self, line='', cell=None, local_ns=None):
         """Time execution of a Python statement or expression
 
-        **Usage, in line mode:**
+        **Usage, in line mode**::
 
-          %timeit [-n<N> -r<R> [-t|-c] -q -p<P> -o] statement
+          %timeit [-n<N> -r<R> [-t|-c] -q -p<P> [-o|-v <V>]] statement
 
-        **or in cell mode:**
-
-          %%timeit [-n<N> -r<R> [-t|-c] -q -p<P> -o] setup_code
+        **or in cell mode**::
 
+          %%timeit [-n<N> -r<R> [-t|-c] -q -p<P> [-o|-v <V>]] setup_code
           code
-
           code...
 
         Time execution of a Python statement or expression using the timeit
@@ -1070,27 +1071,37 @@ class ExecutionMagics(Magics):
 
         Options:
 
-        -n<N>: execute the given statement <N> times in a loop. If <N> is not
-        provided, <N> is determined so as to get sufficient accuracy.
+        -n<N>
+          Execute the given statement N times in a loop. If N is not
+          provided, N is determined so as to get sufficient accuracy.
 
-        -r<R>: number of repeats <R>, each consisting of <N> loops, and take the
-        average result.
-        Default: 7
+        -r<R>
+          Number of repeats R, each consisting of N loops, and take the
+          average result.
+          Default: 7
 
-        -t: use time.time to measure the time, which is the default on Unix.
-        This function measures wall time.
+        -t
+          Use ``time.time`` to measure the time, which is the default on Unix.
+          This function measures wall time.
 
-        -c: use time.clock to measure the time, which is the default on
-        Windows and measures wall time. On Unix, resource.getrusage is used
-        instead and returns the CPU user time.
+        -c
+          Use ``time.clock`` to measure the time, which is the default on
+          Windows and measures wall time. On Unix, ``resource.getrusage`` is used
+          instead and returns the CPU user time.
 
-        -p<P>: use a precision of <P> digits to display the timing result.
-        Default: 3
+        -p<P>
+          Use a precision of P digits to display the timing result.
+          Default: 3
 
-        -q: Quiet, do not print result.
+        -q
+          Quiet, do not print result.
 
-        -o: return a TimeitResult that can be stored in a variable to inspect
-        the result in more details.
+        -o
+          Return a ``TimeitResult`` that can be stored in a variable to inspect
+          the result in more details.
+
+        -v <V>
+          Like ``-o``, but save the ``TimeitResult`` directly to variable <V>.
 
         .. versionchanged:: 7.3
             User variables are no longer expanded,
@@ -1114,27 +1125,29 @@ class ExecutionMagics(Magics):
 
           In [6]: %timeit -n1 time.sleep(2)
 
-        The times reported by %timeit will be slightly higher than those
+        The times reported by ``%timeit`` will be slightly higher than those
         reported by the timeit.py script when variables are accessed. This is
-        due to the fact that %timeit executes the statement in the namespace
+        due to the fact that ``%timeit`` executes the statement in the namespace
         of the shell, compared with timeit.py, which uses a single setup
         statement to import function or create variables. Generally, the bias
         does not matter as long as results from timeit.py are not mixed with
-        those from %timeit."""
+        those from ``%timeit``."""
 
+        # TODO: port to magic_arguments as currently this is duplicated in IPCompleter._extract_code
         opts, stmt = self.parse_options(
-            line, "n:r:tcp:qo", posix=False, strict=False, preserve_non_opts=True
+            line, "n:r:tcp:qov:", posix=False, strict=False, preserve_non_opts=True
         )
         if stmt == "" and cell is None:
             return
-        
+
         timefunc = timeit.default_timer
         number = int(getattr(opts, "n", 0))
         default_repeat = 7 if timeit.default_repeat < 7 else timeit.default_repeat
         repeat = int(getattr(opts, "r", default_repeat))
         precision = int(getattr(opts, "p", 3))
-        quiet = 'q' in opts
-        return_result = 'o' in opts
+        quiet = "q" in opts
+        return_result = "o" in opts
+        save_result = "v" in opts
         if hasattr(opts, "t"):
             timefunc = time.time
         if hasattr(opts, "c"):
@@ -1195,7 +1208,7 @@ class ExecutionMagics(Magics):
                 if var_name in local_ns:
                     conflict_globs[var_name] = var_val
             glob.update(local_ns)
-            
+
         exec(code, glob, ns)
         timer.inner = ns["inner"]
 
@@ -1217,9 +1230,9 @@ class ExecutionMagics(Magics):
 
         # Restore global vars from conflict_globs
         if conflict_globs:
-           glob.update(conflict_globs)
-                
-        if not quiet :
+            glob.update(conflict_globs)
+
+        if not quiet:
             # Check best timing is greater than zero to avoid a
             # ZeroDivisionError.
             # In cases where the slowest timing is lesser than a microsecond
@@ -1229,20 +1242,36 @@ class ExecutionMagics(Magics):
                 print("The slowest run took %0.2f times longer than the "
                       "fastest. This could mean that an intermediate result "
                       "is being cached." % (worst / best))
-           
+
             print( timeit_result )
 
             if tc > tc_min:
                 print("Compiler time: %.2f s" % tc)
+
+        if save_result:
+            self.shell.user_ns[opts.v] = timeit_result
+
         if return_result:
             return timeit_result
 
-    @skip_doctest
     @no_var_expand
+    @magic_arguments.magic_arguments()
+    @magic_arguments.argument(
+        "--no-raise-error",
+        action="store_true",
+        dest="no_raise_error",
+        help="If given, don't re-raise exceptions",
+    )
+    @magic_arguments.kwds(
+        epilog="""
+      Any remaining arguments will be treated as code to run.
+    """
+    )
+    @skip_doctest
     @needs_local_scope
     @line_cell_magic
     @output_can_be_silenced
-    def time(self,line='', cell=None, local_ns=None):
+    def time(self, line="", cell=None, local_ns=None):
         """Time execution of a Python statement or expression.
 
         The CPU and wall clock times are printed, and the value of the
@@ -1257,13 +1286,19 @@ class ExecutionMagics(Magics):
         - In cell mode, you can time the cell body (a directly
           following statement raises an error).
 
-        This function provides very basic timing functionality.  Use the timeit
+        This function provides very basic timing functionality. Use the timeit
         magic for more control over the measurement.
 
         .. versionchanged:: 7.3
             User variables are no longer expanded,
             the magic line is always left unmodified.
 
+        .. versionchanged:: 8.3
+            The time magic now correctly propagates system-exiting exceptions
+            (such as ``KeyboardInterrupt`` invoked when interrupting execution)
+            rather than just printing out the exception traceback.
+            The non-system-exception will still be caught as before.
+
         Examples
         --------
         ::
@@ -1303,11 +1338,12 @@ class ExecutionMagics(Magics):
                 Wall time: 0.00 s
                 Compiler : 0.78 s
         """
-        # fail immediately if the given expression can't be compiled
-        
+        args, extra = magic_arguments.parse_argstring(self.time, line, partial=True)
+        line = " ".join(extra)
+
         if line and cell:
             raise UsageError("Can't use statement directly after '%%time'!")
-        
+
         if cell:
             expr = self.shell.transform_cell(cell)
         else:
@@ -1318,7 +1354,7 @@ class ExecutionMagics(Magics):
 
         t0 = clock()
         expr_ast = self.shell.compile.ast_parse(expr)
-        tp = clock()-t0
+        tp = clock() - t0
 
         # Apply AST transformations
         expr_ast = self.shell.transform_ast(expr_ast)
@@ -1326,8 +1362,8 @@ class ExecutionMagics(Magics):
         # Minimum time above which compilation time will be reported
         tc_min = 0.1
 
-        expr_val=None
-        if len(expr_ast.body)==1 and isinstance(expr_ast.body[0], ast.Expr):
+        expr_val = None
+        if len(expr_ast.body) == 1 and isinstance(expr_ast.body[0], ast.Expr):
             mode = 'eval'
             source = '<timed eval>'
             expr_ast = ast.Expression(expr_ast.body[0].value)
@@ -1336,42 +1372,58 @@ class ExecutionMagics(Magics):
             source = '<timed exec>'
             # multi-line %%time case
             if len(expr_ast.body) > 1 and isinstance(expr_ast.body[-1], ast.Expr):
-                expr_val= expr_ast.body[-1]
+                expr_val = expr_ast.body[-1]
                 expr_ast = expr_ast.body[:-1]
                 expr_ast = Module(expr_ast, [])
                 expr_val = ast.Expression(expr_val.value)
 
         t0 = clock()
         code = self.shell.compile(expr_ast, source, mode)
-        tc = clock()-t0
+        tc = clock() - t0
 
         # skew measurement as little as possible
         glob = self.shell.user_ns
         wtime = time.time
         # time execution
         wall_st = wtime()
-        if mode=='eval':
+        # Track whether to propagate exceptions or exit
+        exit_on_interrupt = False
+        interrupt_occured = False
+        captured_exception = None
+
+        if mode == "eval":
             st = clock2()
             try:
                 out = eval(code, glob, local_ns)
-            except:
-                self.shell.showtraceback()
-                return
+            except KeyboardInterrupt as e:
+                captured_exception = e
+                interrupt_occured = True
+                exit_on_interrupt = True
+            except Exception as e:
+                captured_exception = e
+                interrupt_occured = True
+                if not args.no_raise_error:
+                    exit_on_interrupt = True
             end = clock2()
         else:
             st = clock2()
             try:
                 exec(code, glob, local_ns)
-                out=None
+                out = None
                 # multi-line %%time case
                 if expr_val is not None:
                     code_2 = self.shell.compile(expr_val, source, 'eval')
                     out = eval(code_2, glob, local_ns)
-            except:
-                self.shell.showtraceback()
-                return
+            except KeyboardInterrupt as e:
+                captured_exception = e
+                interrupt_occured = True
+                exit_on_interrupt = True
+            except Exception as e:
+                captured_exception = e
+                interrupt_occured = True
+                if not args.no_raise_error:
+                    exit_on_interrupt = True
             end = clock2()
-
         wall_end = wtime()
         # Compute actual times and report
         wall_time = wall_end - wall_st
@@ -1390,6 +1442,10 @@ class ExecutionMagics(Magics):
             print(f"Compiler : {_format_time(tc)}")
         if tp > tp_min:
             print(f"Parser   : {_format_time(tp)}")
+        if interrupt_occured:
+            if exit_on_interrupt and captured_exception:
+                raise captured_exception
+            return
         return out
 
     @skip_doctest
@@ -1398,18 +1454,21 @@ class ExecutionMagics(Magics):
         """Define a macro for future re-execution. It accepts ranges of history,
         filenames or string objects.
 
-        Usage:\\
+        Usage::
+
           %macro [options] name n1-n2 n3-n4 ... n5 .. n6 ...
 
         Options:
 
-          -r: use 'raw' input.  By default, the 'processed' history is used,
+        -r
+          Use 'raw' input.  By default, the 'processed' history is used,
           so that magics are loaded in their transformed version to valid
           Python.  If this option is given, the raw input as typed at the
           command line is used instead.
-          
-          -q: quiet macro definition.  By default, a tag line is printed 
-          to indicate the macro has been created, and then the contents of 
+
+        -q
+          Quiet macro definition.  By default, a tag line is printed
+          to indicate the macro has been created, and then the contents of
           the macro are printed.  If this option is given, then no printout
           is produced once the macro is created.
 
@@ -1471,31 +1530,41 @@ class ExecutionMagics(Magics):
             return
         macro = Macro(lines)
         self.shell.define_macro(name, macro)
-        if not ( 'q' in opts) : 
-            print('Macro `%s` created. To execute, type its name (without quotes).' % name)
-            print('=== Macro contents: ===')
-            print(macro, end=' ')
+        if "q" not in opts:
+            print(
+                "Macro `%s` created. To execute, type its name (without quotes)." % name
+            )
+            print("=== Macro contents: ===")
+            print(macro, end=" ")
 
     @magic_arguments.magic_arguments()
-    @magic_arguments.argument('output', type=str, default='', nargs='?',
-        help="""The name of the variable in which to store output.
-        This is a utils.io.CapturedIO object with stdout/err attributes
+    @magic_arguments.argument(
+        "output",
+        type=str,
+        default="",
+        nargs="?",
+        help="""
+
+        The name of the variable in which to store output.
+        This is a ``utils.io.CapturedIO`` object with stdout/err attributes
         for the text of the captured output.
 
-        CapturedOutput also has a show() method for displaying the output,
-        and __call__ as well, so you can use that to quickly display the
+        CapturedOutput also has a ``show()`` method for displaying the output,
+        and ``__call__`` as well, so you can use that to quickly display the
         output.
 
         If unspecified, captured output is discarded.
-        """
+        """,
     )
-    @magic_arguments.argument('--no-stderr', action="store_true",
-        help="""Don't capture stderr."""
+    @magic_arguments.argument(
+        "--no-stderr", action="store_true", help="""Don't capture stderr."""
     )
-    @magic_arguments.argument('--no-stdout', action="store_true",
-        help="""Don't capture stdout."""
+    @magic_arguments.argument(
+        "--no-stdout", action="store_true", help="""Don't capture stdout."""
     )
-    @magic_arguments.argument('--no-display', action="store_true",
+    @magic_arguments.argument(
+        "--no-display",
+        action="store_true",
         help="""Don't capture IPython's rich display."""
     )
     @cell_magic
@@ -1597,26 +1666,26 @@ def parse_breakpoint(text, current_file):
         return current_file, int(text)
     else:
         return text[:colon], int(text[colon+1:])
-    
+
+
 def _format_time(timespan, precision=3):
     """Formats the timespan in a human readable form"""
 
     if timespan >= 60.0:
         # we have more than a minute, format that in a human readable form
         # Idea from http://snipplr.com/view/5713/
-        parts = [("d", 60*60*24),("h", 60*60),("min", 60), ("s", 1)]
+        parts = [("d", 60 * 60 * 24), ("h", 60 * 60), ("min", 60), ("s", 1)]
         time = []
         leftover = timespan
         for suffix, length in parts:
             value = int(leftover / length)
             if value > 0:
                 leftover = leftover % length
-                time.append(u'%s%s' % (str(value), suffix))
+                time.append("%s%s" % (str(value), suffix))
             if leftover < 1:
                 break
         return " ".join(time)
 
-
     # Unfortunately characters outside of range(128) can cause problems in
     # certain terminals.
     # See bug: https://bugs.launchpad.net/ipython/+bug/348466
@@ -1630,7 +1699,7 @@ def _format_time(timespan, precision=3):
         except:
             pass
     scaling = [1, 1e3, 1e6, 1e9]
-        
+
     if timespan > 0.0:
         order = min(-int(math.floor(math.log10(timespan)) // 3), 3)
     else:
