@@ -9,7 +9,7 @@ reference the name under which an object is being read.
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
 
-__all__ = ['Inspector','InspectColors']
+__all__ = ["Inspector"]
 
 # stdlib modules
 from dataclasses import dataclass
@@ -23,6 +23,7 @@ import linecache
 import os
 import types
 import warnings
+from pygments.token import Token
 
 
 from typing import (
@@ -38,6 +39,7 @@ from typing import (
 )
 
 import traitlets
+from traitlets.config import Configurable
 
 # IPython's own
 from IPython.core import page
@@ -48,8 +50,6 @@ from IPython.utils.dir2 import safe_hasattr
 from IPython.utils.path import compress_user
 from IPython.utils.text import indent
 from IPython.utils.wildcard import list_namespace, typestr2type
-from IPython.utils.coloransi import TermColors
-from IPython.utils.colorable import Colorable
 from IPython.utils.decorators import undoc
 
 from pygments import highlight
@@ -79,13 +79,14 @@ class OInfo:
         """
         # We need to deprecate this at some point, but the warning will show in completion.
         # Let's comment this for now and uncomment end of 2023 ish
-        #        warnings.warn(
-        #            f"OInfo dataclass with fields access since IPython 8.12 please use OInfo.{field} instead."
-        #            "OInfo used to be a dict but a dataclass provide static fields verification with mypy."
-        #            "This warning and backward compatibility `get()` method were added in 8.13.",
-        #            DeprecationWarning,
-        #            stacklevel=2,
-        #        )
+        # Jan 2025: decomenting for IPython 9.0
+        warnings.warn(
+            f"OInfo dataclass with fields access since IPython 8.12 please use OInfo.{field} instead."
+            "OInfo used to be a dict but a dataclass provide static fields verification with mypy."
+            "This warning and backward compatibility `get()` method were added in 8.13.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         return getattr(self, field)
 
 
@@ -105,9 +106,6 @@ _builtin_meth_type = type(str.upper)  # Bound methods have the same type as buil
 #****************************************************************************
 # Builtin color schemes
 
-Colors = TermColors  # just a shorthand
-
-InspectColors = PyColorize.ANSICodeColors
 
 #****************************************************************************
 # Auxiliary functions and objects
@@ -175,8 +173,8 @@ def object_info(
     **kw,
 ) -> InfoDict:
     """Make an object info dict with all fields present."""
-    infodict = kw
-    infodict = {k: None for k in _info_fields if k not in infodict}
+    infodict = dict(kw)
+    infodict.update({k: None for k in _info_fields if k not in infodict})
     infodict["name"] = name  # type: ignore
     infodict["found"] = found  # type: ignore
     infodict["isclass"] = isclass  # type: ignore
@@ -255,7 +253,6 @@ def getsource(obj, oname='') -> Union[str,None]:
         for attrname in ['fget', 'fset', 'fdel']:
             fn = getattr(obj, attrname)
             if fn is not None:
-                encoding = get_encoding(fn)
                 oname_prefix = ('%s.' % oname) if oname else ''
                 sources.append(''.join(('# ', oname_prefix, attrname)))
                 if inspect.isfunction(fn):
@@ -299,79 +296,6 @@ def is_simple_callable(obj):
     return (inspect.isfunction(obj) or inspect.ismethod(obj) or \
             isinstance(obj, _builtin_func_type) or isinstance(obj, _builtin_meth_type))
 
-@undoc
-def getargspec(obj):
-    """Wrapper around :func:`inspect.getfullargspec`
-
-    In addition to functions and methods, this can also handle objects with a
-    ``__call__`` attribute.
-
-    DEPRECATED: Deprecated since 7.10. Do not use, will be removed.
-    """
-
-    warnings.warn('`getargspec` function is deprecated as of IPython 7.10'
-                  'and will be removed in future versions.', DeprecationWarning, stacklevel=2)
-
-    if safe_hasattr(obj, '__call__') and not is_simple_callable(obj):
-        obj = obj.__call__
-
-    return inspect.getfullargspec(obj)
-
-@undoc
-def format_argspec(argspec):
-    """Format argspect, convenience wrapper around inspect's.
-
-    This takes a dict instead of ordered arguments and calls
-    inspect.format_argspec with the arguments in the necessary order.
-
-    DEPRECATED (since 7.10): Do not use; will be removed in future versions.
-    """
-    
-    warnings.warn('`format_argspec` function is deprecated as of IPython 7.10'
-                  'and will be removed in future versions.', DeprecationWarning, stacklevel=2)
-
-
-    return inspect.formatargspec(argspec['args'], argspec['varargs'],
-                                 argspec['varkw'], argspec['defaults'])
-
-@undoc
-def call_tip(oinfo, format_call=True):
-    """DEPRECATED since 6.0. Extract call tip data from an oinfo dict."""
-    warnings.warn(
-        "`call_tip` function is deprecated as of IPython 6.0"
-        "and will be removed in future versions.",
-        DeprecationWarning,
-        stacklevel=2,
-    )
-    # Get call definition
-    argspec = oinfo.get('argspec')
-    if argspec is None:
-        call_line = None
-    else:
-        # Callable objects will have 'self' as their first argument, prune
-        # it out if it's there for clarity (since users do *not* pass an
-        # extra first argument explicitly).
-        try:
-            has_self = argspec['args'][0] == 'self'
-        except (KeyError, IndexError):
-            pass
-        else:
-            if has_self:
-                argspec['args'] = argspec['args'][1:]
-
-        call_line = oinfo['name']+format_argspec(argspec)
-
-    # Now get docstring.
-    # The priority is: call docstring, constructor docstring, main one.
-    doc = oinfo.get('call_docstring')
-    if doc is None:
-        doc = oinfo.get('init_docstring')
-    if doc is None:
-        doc = oinfo.get('docstring','')
-
-    return call_line, doc
-
-
 def _get_wrapped(obj):
     """Get the original object if wrapped in one or more @decorators
 
@@ -456,28 +380,41 @@ def find_source_lines(obj):
 
     return lineno
 
-class Inspector(Colorable):
 
+_sentinel = object()
+
+
+class Inspector(Configurable):
     mime_hooks = traitlets.Dict(
         config=True,
-        help="dictionary of mime to callable to add informations into help mimebundle dict",
+        help="dictionary of mime to callable to add information into help mimebundle dict",
     ).tag(config=True)
 
+    _theme_name: str
+
     def __init__(
         self,
-        color_table=InspectColors,
-        code_color_table=PyColorize.ANSICodeColors,
-        scheme=None,
+        *,
+        theme_name: str,
         str_detail_level=0,
         parent=None,
         config=None,
     ):
+        if theme_name in ["Linux", "LightBG", "Neutral", "NoColor"]:
+            warnings.warn(
+                f"Theme names and color schemes are lowercase in IPython 9.0 use {theme_name.lower()} instead",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+            theme_name = theme_name.lower()
+        self._theme_name = theme_name
         super(Inspector, self).__init__(parent=parent, config=config)
-        self.color_table = color_table
-        self.parser = PyColorize.Parser(out='str', parent=self, style=scheme)
-        self.format = self.parser.format
+        self.parser = PyColorize.Parser(out="str", theme_name=theme_name)
         self.str_detail_level = str_detail_level
-        self.set_active_scheme(scheme)
+        self.set_theme_name(theme_name)
+
+    def format(self, *args, **kwargs):
+        return self.parser.format(*args, **kwargs)
 
     def _getdef(self,obj,oname='') -> Union[str,None]:
         """Return the call signature for any callable object.
@@ -491,15 +428,26 @@ class Inspector(Colorable):
         except:
             return None
 
-    def __head(self,h) -> str:
+    def __head(self, h: str) -> str:
         """Return a header string with proper colors."""
-        return '%s%s%s' % (self.color_table.active_colors.header,h,
-                           self.color_table.active_colors.normal)
+        return PyColorize.theme_table[self._theme_name].format([(Token.Header, h)])
 
-    def set_active_scheme(self, scheme):
-        if scheme is not None:
-            self.color_table.set_active_scheme(scheme)
-            self.parser.color_table.set_active_scheme(scheme)
+    def set_theme_name(self, name: str):
+        assert name == name.lower()
+        assert name in PyColorize.theme_table.keys()
+        self._theme_name = name
+        self.parser.theme_name = name
+
+    def set_active_scheme(self, scheme: str):
+        warnings.warn(
+            "set_active_scheme is deprecated and replaced by set_theme_name as of IPython 9.0",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        assert scheme == scheme.lower()
+        if scheme is not None and self._theme_name != scheme:
+            self._theme_name = scheme
+            self.parser.theme_name = scheme
 
     def noinfo(self, msg, oname):
         """Generic message when no information is found."""
@@ -565,23 +513,22 @@ class Inspector(Colorable):
         No documentation found for obj2
         """
 
-        head = self.__head  # For convenience
         lines = []
         ds = getdoc(obj)
         if formatter:
             ds = formatter(ds).get('plain/text', ds)
         if ds:
-            lines.append(head("Class docstring:"))
+            lines.append(self.__head("Class docstring:"))
             lines.append(indent(ds))
         if inspect.isclass(obj) and hasattr(obj, '__init__'):
             init_ds = getdoc(obj.__init__)
             if init_ds is not None:
-                lines.append(head("Init docstring:"))
+                lines.append(self.__head("Init docstring:"))
                 lines.append(indent(init_ds))
         elif hasattr(obj,'__call__'):
             call_ds = getdoc(obj.__call__)
             if call_ds:
-                lines.append(head("Call docstring:"))
+                lines.append(self.__head("Call docstring:"))
                 lines.append(indent(call_ds))
 
         if not lines:
@@ -691,7 +638,7 @@ class Inspector(Colorable):
             if isinstance(bundle["text/html"], (list, tuple)):
                 # bundle['text/html'] is a list of (head, formatted body) pairs
                 new_b["text/html"] = "\n".join(
-                    (f"<h1>{head}</h1>\n{body}" for (head, body) in bundle["text/html"])
+                    f"<h1>{head}</h1>\n{body}" for (head, body) in bundle["text/html"]
                 )
 
         for k in bundle.keys():
@@ -904,21 +851,6 @@ class Inspector(Colorable):
             del info_b["text/html"]
         page.page(info_b)
 
-    def _info(self, obj, oname="", info=None, detail_level=0):
-        """
-        Inspector.info() was likely improperly marked as deprecated
-        while only a parameter was deprecated. We "un-deprecate" it.
-        """
-
-        warnings.warn(
-            "The `Inspector.info()` method has been un-deprecated as of 8.0 "
-            "and the `formatter=` keyword removed. `Inspector._info` is now "
-            "an alias, and you can just call `.info()` directly.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        return self.info(obj, oname=oname, info=info, detail_level=detail_level)
-
     def info(self, obj, oname="", info=None, detail_level=0) -> InfoDict:
         """Compute a dict with detailed information about an object.
 
@@ -1196,7 +1128,7 @@ class Inspector(Colorable):
 
           - list_types(False): list all available object types for object matching.
         """
-        #print 'ps pattern:<%r>' % pattern # dbg
+        # print('ps pattern:<%r>' % pattern)  # dbg
 
         # defaults
         type_pattern = 'all'
@@ -1225,7 +1157,7 @@ class Inspector(Colorable):
                 raise ValueError('invalid namespace <%s>. Valid names: %s' %
                                  (name,ns_table.keys()))
 
-        #print 'type_pattern:',type_pattern # dbg
+        # print('type_pattern:',type_pattern)  # dbg
         search_result, namespaces_seen = set(), set()
         for ns_name in ns_search:
             ns = ns_table[ns_name]
