@@ -136,7 +136,7 @@ The following is a very simple example of a valid demo file.
     #################### EXAMPLE DEMO <ex_demo.py> ###############################
     '''A simple interactive demo to illustrate the use of IPython's Demo class.'''
 
-    print 'Hello, welcome to an interactive IPython demo.'
+    print('Hello, welcome to an interactive IPython demo.')
 
     # The mark below defines a block boundary, which is a point where IPython will
     # stop execution and return to the interactive prompt. The dashes are actually
@@ -152,21 +152,21 @@ The following is a very simple example of a valid demo file.
     # the mark below makes this block as silent
     # <demo> silent
 
-    print 'This is a silent block, which gets executed but not printed.'
+    print('This is a silent block, which gets executed but not printed.')
 
     # <demo> stop
     # <demo> auto
-    print 'This is an automatic block.'
-    print 'It is executed without asking for confirmation, but printed.'
-    z = x+y
+    print('This is an automatic block.')
+    print('It is executed without asking for confirmation, but printed.')
+    z = x + y
 
-    print 'z=',x
+    print('z =', x)
 
     # <demo> stop
     # This is just another normal block.
-    print 'z is now:', z
+    print('z is now:', z)
 
-    print 'bye!'
+    print('bye!')
     ################### END EXAMPLE DEMO <ex_demo.py> ############################
 """
 
@@ -184,6 +184,7 @@ import re
 import shlex
 import sys
 import pygments
+from pathlib import Path
 
 from IPython.utils.text import marquee
 from IPython.utils import openpy
@@ -195,7 +196,7 @@ class DemoError(Exception): pass
 def re_mark(mark):
     return re.compile(r'^\s*#\s+<demo>\s+%s\s*$' % mark,re.MULTILINE)
 
-class Demo(object):
+class Demo:
 
     re_stop     = re_mark(r'-*\s?stop\s?-*')
     re_silent   = re_mark('silent')
@@ -238,7 +239,7 @@ class Demo(object):
             terminal16m
 
           - style('default'): a string of pygments style name to be used.
-          """
+        """
         if hasattr(src, "read"):
              # It seems to be a file or a file-like object
             self.fname = "from a file-like object"
@@ -403,8 +404,8 @@ class Demo(object):
             index -= 1
 
         filename = self.shell.mktempfile(self.src_blocks[index])
-        self.shell.hooks.editor(filename,1)
-        with open(filename, 'r') as f:
+        self.shell.hooks.editor(filename, 1)
+        with open(Path(filename), "r", encoding="utf-8") as f:
             new_block = f.read()
         # update the source and colored block
         self.src_blocks[index] = new_block
@@ -531,7 +532,7 @@ class Demo(object):
                 elif token[0] == Token.Comment.Single:
                     toks.append((Token.Comment.Single, token[1][0]))
                     # parse comment content by rst lexer
-                    # remove the extrat newline added by rst lexer
+                    # remove the extra newline added by rst lexer
                     toks += list(pygments.lex(token[1][1:], self.rst_lexer))[:-1]
                 else:
                     toks.append(token)
@@ -597,7 +598,7 @@ class IPythonLineDemo(IPythonDemo,LineDemo):
     pass
 
 
-class ClearMixin(object):
+class ClearMixin:
     """Use this mixin to make Demo classes with less visual clutter.
 
     Demos using this mixin will clear the screen before every block and use
