@@ -127,14 +127,6 @@ def _safe_getattr(obj, attr, default=None):
     except Exception:
         return default
 
-@undoc
-class CUnicodeIO(StringIO):
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        warn(("CUnicodeIO is deprecated since IPython 6.0. "
-              "Please use io.StringIO instead."),
-             DeprecationWarning, stacklevel=2)
-
 def _sorted_for_pprint(items):
     """
     Sort the given items for pretty printing. Since some predictable
@@ -171,7 +163,8 @@ def pprint(obj, verbose=False, max_width=79, newline='\n', max_seq_length=MAX_SE
     sys.stdout.write(newline)
     sys.stdout.flush()
 
-class _PrettyPrinterBase(object):
+
+class _PrettyPrinterBase:
 
     @contextmanager
     def indent(self, indent):
@@ -442,7 +435,7 @@ class RepresentationPrinter(PrettyPrinter):
         return printer
 
 
-class Printable(object):
+class Printable:
 
     def output(self, stream, output_width):
         return output_width
@@ -494,7 +487,7 @@ class Group(Printable):
         self.want_break = False
 
 
-class GroupQueue(object):
+class GroupQueue:
 
     def __init__(self, *groups):
         self.queue = []
@@ -947,7 +940,8 @@ for_type_by_name("collections", "UserList", _userlist_pprint)
 
 if __name__ == '__main__':
     from random import randrange
-    class Foo(object):
+
+    class Foo:
         def __init__(self):
             self.foo = 1
             self.bar = re.compile(r'\s+')
