@@ -2,7 +2,8 @@ import re
 import asyncio
 import tokenize
 from io import StringIO
-from typing import Callable, List, Optional, Union, Generator, Tuple, ClassVar, Any
+from typing import List, Optional, Union, Tuple, ClassVar, Any
+from collections.abc import Callable, Generator
 import warnings
 
 import prompt_toolkit
@@ -186,7 +187,7 @@ class NavigableAutoSuggestFromHistory(AutoSuggestFromHistory):
         self._init_llm_provider = None
         self._request_number = 0
 
-    def reset_history_position(self, _: Buffer):
+    def reset_history_position(self, _: Buffer) -> None:
         self.skip_lines = 0
 
     def disconnect(self) -> None:
@@ -195,7 +196,7 @@ class NavigableAutoSuggestFromHistory(AutoSuggestFromHistory):
             text_insert_event = pt_app.default_buffer.on_text_insert
             text_insert_event.remove_handler(self.reset_history_position)
 
-    def connect(self, pt_app: PromptSession):
+    def connect(self, pt_app: PromptSession) -> None:
         self._connected_apps.append(pt_app)
         # note: `on_text_changed` could be used for a bit different behaviour
         # on character deletion (i.e. resetting history position on backspace)
@@ -473,15 +474,6 @@ def accept_or_jump_to_end(event: KeyPressEvent):
         nc.end_of_line(event)
 
 
-def _deprected_accept_in_vi_insert_mode(event: KeyPressEvent):
-    """Accept autosuggestion or jump to end of line.
-
-    .. deprecated:: 8.12
-        Use `accept_or_jump_to_end` instead.
-    """
-    return accept_or_jump_to_end(event)
-
-
 def accept(event: KeyPressEvent):
     """Accept autosuggestion"""
     buffer = event.current_buffer
@@ -664,16 +656,3 @@ def swap_autosuggestion_down(event: KeyPressEvent):
         provider=provider,
         direction_method=provider.down,
     )
-
-
-def __getattr__(key):
-    if key == "accept_in_vi_insert_mode":
-        warnings.warn(
-            "`accept_in_vi_insert_mode` is deprecated since IPython 8.12 and "
-            "renamed to `accept_or_jump_to_end`. Please update your configuration "
-            "accordingly",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        return _deprected_accept_in_vi_insert_mode
-    raise AttributeError
