@@ -4,6 +4,7 @@
 #
 # Copyright (c) 2004-2021 Holger Krekel and others
 """Discover and run ipdoctests in modules and test files."""
+
 import bdb
 import builtins
 import inspect
@@ -18,18 +19,15 @@ from pathlib import Path
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
     Dict,
-    Generator,
-    Iterable,
     List,
     Optional,
-    Pattern,
-    Sequence,
     Tuple,
     Type,
     Union,
 )
+from re import Pattern
+from collections.abc import Callable, Generator, Iterable, Sequence
 
 import pytest
 from _pytest import outcomes
@@ -349,7 +347,7 @@ class IPDoctestItem(pytest.Item):
         assert self.runner is not None
         _check_all_skipped(self.dtest)
         self._disable_output_capturing_for_darwin()
-        failures: List["doctest.DocTestFailure"] = []
+        failures: List[doctest.DocTestFailure] = []
 
         # exec(compile(..., "single", ...), ...) puts result in builtins._
         had_underscore_value = hasattr(builtins, "_")
